package gui;

import java.awt.BorderLayout;
import java.awt.Font;
import javax.swing.*;
import jview.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PiePlot;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.data.general.PieDataset;

/**
 *
 * @author Lou
 */
public class PieChartViewPanel extends JPanel
{
    private JComboBox viewBy;
    private JPanel chartPanel;
    
    PieChartViewPanel()
    {
        this.setLayout(new BorderLayout());
            
        viewBy = new JComboBox(new String[]{"- Select", "LoC", "Properties", "Methods", "Couplings"});
        viewBy.setBounds(5, 5, 150, 25);

        viewBy.addActionListener (new ActionListener ()
        {
            public void actionPerformed(ActionEvent ae)
            {
                viewByValueChangedActionPerformed(ae);
            }
        });

        add(viewBy, BorderLayout.PAGE_START);

        chartPanel = new JPanel();
        this.add(chartPanel);
    }

    private JFreeChart createChart(String name, PieDataset dataset) {
        
        JFreeChart chart = ChartFactory.createPieChart(
            name, 
            dataset,
            false,
            true,
            false
        );
        
        PiePlot plot = (PiePlot) chart.getPlot();
        plot.setLabelFont(new Font("SansSerif", Font.PLAIN, 12));
        plot.setNoDataMessage("No data available");
        plot.setCircular(true);
        plot.setLabelGap(0.02);
        return chart;
    }
    
    private PieDataset createDataset(ArrayList<Tag> tags)
    {
        DefaultPieDataset dataset = new DefaultPieDataset();
        
        for (Tag t: tags)
            dataset.setValue(t.getName(), t.getWeight());
        
        return dataset;        
    }
    
    private void viewByValueChangedActionPerformed(ActionEvent ae)
    {
        ArrayList<Tag> tags = new ArrayList<Tag>();
        if(viewBy.getSelectedItem().toString().equals("Properties"))
        {
            for (JavaClass c: Project.getInstance().getClasses())
                tags.add(new Tag(c.getFullName(), c.getProperties().size()));
        }
        else if(viewBy.getSelectedItem().toString().equals("Methods"))
        {
            for (JavaClass c: Project.getInstance().getClasses())
                tags.add(new Tag(c.getFullName(), c.getMethods().size()));
        }
        else if(viewBy.getSelectedItem().toString().equals("LoC"))
        {
            for (JavaClass c: Project.getInstance().getClasses())
                tags.add(new Tag(c.getFullName(), c.getLoc()));
        }
        else if(viewBy.getSelectedItem().toString().equals("Couplings"))
        {
            for (JavaClass c: Project.getInstance().getClasses())
                if(c.hasEfferentCouplings())
                {
                    tags.add(new Tag(c.getFullName(), c.getRelationsTo().size()));
                }
        }
        else
        {
            this.remove(chartPanel);
            chartPanel = new JPanel();
            this.add(chartPanel);
            this.repaint();
            this.revalidate();
            return;
        }

        this.remove(chartPanel);
        chartPanel = new ChartPanel(createChart(viewBy.getSelectedItem().toString(), createDataset(tags)));
        this.add(chartPanel, BorderLayout.CENTER);
        this.repaint();
        this.revalidate();
    }
}
