/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package gui;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.Dimension;
import jview.JavaClass;
import jview.JavaMethod;
import jview.JavaProperty;
import jview.Project;

/**
 * @author Lou
 */
public class SimpleViewClassesPanel extends JPanel
{
    private JTable classesTable;
    private JTable propertiesTable;
    private JTable methodsTable;
    private JLabel classesLabel;
    private JLabel propertiesLabel;
    private JLabel methodsLabel;
    private DefaultTableModel propertiesTableModel;
    private DefaultTableModel methodsTableModel;

    SimpleViewClassesPanel()
    {
        setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

        classesLabel = new JLabel("Classes");
        classesLabel.setPreferredSize(new Dimension(100, 40));
        
        //DefaultTableModel for the Classes of the Project
        DefaultTableModel classesTableModel = new DefaultTableModel() {
            Class[] types = { String.class, String.class, String.class, String.class, Integer.class, Integer.class, Integer.class, Integer.class , Integer.class };

            @Override
            public boolean isCellEditable(int row, int column) {
                return false;
            }

            @Override
             public Class getColumnClass(int columnIndex) {
                return this.types[columnIndex];
             }
        };

        classesTableModel.addColumn("Class");
        classesTableModel.addColumn("Package");
        classesTableModel.addColumn("Parent");
        classesTableModel.addColumn("Implements");
        classesTableModel.addColumn("LOC");
        classesTableModel.addColumn("Properties");
        classesTableModel.addColumn("Methods");
        classesTableModel.addColumn("Ce");
        classesTableModel.addColumn("Ca");
        
        for (JavaClass c: Project.getInstance().getClasses())
            classesTableModel.addRow(new Object[]{c.getName(), c.getPackageName(), c.getParentClass(), c.getImplementsAsString(), c.getLoc(), c.getProperties().size(), c.getMethods().size(), c.getRelationsTo().size(), c.getRelationsFrom().size()});

        classesTable = new JTable(classesTableModel);
        classesTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        classesTable.setRowSorter(new TableRowSorter<TableModel>(classesTable.getModel()));

        propertiesLabel = new JLabel("Properties");
        propertiesLabel.setPreferredSize(new Dimension(100, 40));

        //DefaultTableModel for the properties of the selected class
        propertiesTableModel = new DefaultTableModel()
        {
            @Override
            public boolean isCellEditable(int row, int column)
            {
                return false;
            }
        };

        propertiesTableModel.addColumn("Name");
        propertiesTableModel.addColumn("Type");
        propertiesTableModel.addColumn("Access");
        propertiesTableModel.addColumn("Static");
        propertiesTableModel.addColumn("Final");

        //propertiesTableModel.addRow(new Object[]{"", "", "", "", ""});
        propertiesTable = new JTable(propertiesTableModel);
        propertiesTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        propertiesTable.setRowSorter(new TableRowSorter<TableModel>(propertiesTable.getModel()));

        methodsLabel = new JLabel("Methods");
        methodsLabel.setPreferredSize(new Dimension(100, 40));

        //DefaultTableModel for the methods of the selected class
        methodsTableModel = new DefaultTableModel()
        {
            @Override
            public boolean isCellEditable(int row, int column)
            {
                return false;
            }
        };

        methodsTableModel.addColumn("Name");
        methodsTableModel.addColumn("Returns");
        methodsTableModel.addColumn("Arguments");
        methodsTableModel.addColumn("Abstract");
        methodsTableModel.addColumn("Static");
        methodsTableModel.addColumn("Final");
        methodsTableModel.addColumn("Constructor");

        methodsTable = new JTable(methodsTableModel);
        methodsTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        methodsTable.setRowSorter(new TableRowSorter<TableModel>(methodsTable.getModel()));

        //List event
        classesTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
        {
            public void valueChanged(ListSelectionEvent lse)
            {
                if (!lse.getValueIsAdjusting())
                {
                    for (JavaClass c: Project.getInstance().getClasses())
                    {
                        if (c.getName().equals(classesTable.getValueAt(classesTable.getSelectedRow(), 0).toString()) && c.getPackageName().equals(classesTable.getValueAt(classesTable.getSelectedRow(), 1).toString()))
                        {
                            for (int i = propertiesTableModel.getRowCount() - 1; i > -1; i--)
                                propertiesTableModel.removeRow(i);

                            for (int i = methodsTableModel.getRowCount() - 1; i > -1; i--)
                                methodsTableModel.removeRow(i);

                            if (c.getProperties().size() > 0)
                            {
                                for (JavaProperty jp: c.getProperties())
                                    propertiesTableModel.addRow(new Object[]{jp.getName(), jp.getType(), jp.getAccess(), jp.getIsStatic() , jp.getIsFinal()});
                            }

                            if (c.getMethods().size() > 0)
                            {
                                for (JavaMethod jm: c.getMethods())
                                    methodsTableModel.addRow(new Object[]{jm.getName(), jm.getReturnType(), jm.getArgumentTypesAsString(), jm.getIsAbstract(), jm.getIsStatic(), jm.getIsFinal(), jm.getIsConstructor()});
                            }
                            
                            break;
                        }
                    }
                }
            }
        });

        //Classes
        add(classesLabel);
        add(new JScrollPane(classesTable));

        //Properties
        add(propertiesLabel);
        add(new JScrollPane(propertiesTable));

        //Methods
        add(methodsLabel);
        add(new JScrollPane(methodsTable));
    }
}
