/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package gui;

import java.awt.Dimension;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import jview.JavaInterface;
import jview.JavaMethod;
import jview.Project;

/**
 *
 * @author Lou
 */
public class SimpleViewInterfacesPanel extends JPanel
{
    private JTable interfacesTable;
    private JTable methodsTable;
    private JLabel interfacesLabel;
    private JLabel methodsLabel;
    private DefaultTableModel methodsTableModel;

    SimpleViewInterfacesPanel()
    {
        setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

        interfacesLabel = new JLabel("Interfaces");
        interfacesLabel.setPreferredSize(new Dimension(100, 30));

        //DefaultTableModel for the Classes of the Project
        DefaultTableModel interfacesTableModel = new DefaultTableModel()
        {
            Class[] types = { String.class, String.class, String.class, Integer.class, Integer.class };

            @Override
            public boolean isCellEditable(int row, int column)
            {
                return false;
            }

            @Override
            public Class getColumnClass(int columnIndex)
            {
                return this.types[columnIndex];
            }
        };

        interfacesTableModel.addColumn("Interface");
        interfacesTableModel.addColumn("Package");
        interfacesTableModel.addColumn("Parents");
        interfacesTableModel.addColumn("LOC");
        interfacesTableModel.addColumn("Methods");

        for (JavaInterface i: Project.getInstance().getInterfaces())
            interfacesTableModel.addRow(new Object[]{i.getName(), i.getPackageName(), i.parentInterfacesAsString(), i.getLoc(), i.getMethods().size()});

        interfacesTable = new JTable(interfacesTableModel);
        interfacesTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        interfacesTable.setRowSorter(new TableRowSorter<TableModel>(interfacesTable.getModel()));

        methodsLabel = new JLabel("Methods");
        methodsLabel.setPreferredSize(new Dimension(100, 30));

        //DefaultTableModel for the methods of the selected class
        methodsTableModel = new DefaultTableModel()
        {
            @Override
            public boolean isCellEditable(int row, int column)
            {
                return false;
            }
        };

        methodsTableModel.addColumn("Name");
        methodsTableModel.addColumn("Returns");
        methodsTableModel.addColumn("Arguments");

        methodsTable = new JTable(methodsTableModel);
        methodsTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        methodsTable.setRowSorter(new TableRowSorter<TableModel>(methodsTable.getModel()));

        //List event
        interfacesTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
        {
            public void valueChanged(ListSelectionEvent lse)
            {
                if (!lse.getValueIsAdjusting())
                {
                    for (JavaInterface c: Project.getInstance().getInterfaces())
                    {
                        if (c.getName().equals(interfacesTable.getValueAt(interfacesTable.getSelectedRow(), 0).toString()) && c.getPackageName().equals(interfacesTable.getValueAt(interfacesTable.getSelectedRow(), 1).toString()))
                        {
                            for (int i = methodsTableModel.getRowCount() - 1; i > -1; i--)
                                methodsTableModel.removeRow(i);

                            if (c.getMethods().size() > 0)
                            {
                                for (JavaMethod jm: c.getMethods())
                                    methodsTableModel.addRow(new Object[]{jm.getName(), jm.getReturnType(), jm.getArgumentTypesAsString()});
                            }

                            break;
                        }
                    }
                }
            }
        });

        //Interfaces
        add(interfacesLabel);
        add(new JScrollPane(interfacesTable));
        
        //Methods
        add(methodsLabel);
        add(new JScrollPane(methodsTable));
    }
}
