/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package jview;

import java.util.ArrayList;

/**
 *
 * @author Administrator
 */
public abstract class JavaEntity
{
    private String name;
    private String fullName;
    private String packageName;
    private ArrayList<String> imports;
    private ArrayList<JavaProperty> properties;
    private ArrayList<JavaMethod> methods;
    private ArrayList<JavaEntity> internalEntities;
    private ArrayList<Relation> relationsTo;
    private ArrayList<Relation> relationsFrom;
    private String code;
    private int loc;
    
    JavaEntity()
    {   
        imports = new ArrayList<String>();
        properties = new ArrayList<JavaProperty>();
        methods = new ArrayList<JavaMethod>();
        internalEntities = new ArrayList<JavaEntity>();
        relationsTo = new ArrayList<Relation>();
        relationsFrom = new ArrayList<Relation>();
    }

    public void addProperty(JavaProperty property)
    {
        this.properties.add(property);
    }

    public ArrayList<JavaProperty> getProperties()
    {
        return properties;
    }
   
    public void addMethod(JavaMethod method)
    {
        this.methods.add(method);
    }

    public ArrayList<JavaMethod> getMethods()
    {
        return methods;
    }

    public void addInternalEntity(JavaEntity internalEntity)
    {
        this.internalEntities.add(internalEntity);
    }

    public ArrayList<JavaEntity> getInternalEntities()
    {
        return internalEntities;
    }
    
    public void setImports(ArrayList<String> imports)
    {
        this.imports = imports;
    }

    public ArrayList<String> getImports()
    {
        return imports;
    }

    public ArrayList<Relation> getRelationsTo()
    {
        return relationsTo;
    }
    
    public ArrayList<Relation> getUniqueRelationsTo()
    {
        ArrayList<Relation> unique = new ArrayList<Relation>();
        for (Relation r: this.relationsTo)
        {
            boolean flag = false;
            for (Relation j: unique)
            {
                if (j.getRelatedEntity().equals(r.getRelatedEntity()))
                {
                    flag = true;
                    break;
                }
            }
            
            if (flag == false)
                unique.add(r);
        }
        
        return unique;
    }

    public int countRelationsTo(JavaEntity entity)
    {
        int count = 0;

        for(Relation r : this.relationsTo)
            if(r.getRelatedEntity().equals(entity.getFullName()))
                count++;

        return count;
    }
    
    public void addRelationTo(Relation relationTo)
    {
        this.relationsTo.add(relationTo);
    }
    
    public ArrayList<Relation> getRelationsFrom()
    {
        return relationsFrom;
    }
    
    public void addRelationFrom(Relation relationFrom)
    {
        this.relationsFrom.add(relationFrom);
    }
    
    public void setName(String name)
    {
        this.name = name;
    }

    public String getName()
    {
        return name;
    }

    public void setFullName(String fullName)
    {
        this.fullName = fullName;
    }

    public String getFullName()
    {
        return fullName;
    }

    public void setPackageName(String packageName)
    {
        this.packageName = packageName;
    }
    
    public String getPackageName()
    {
        return packageName;
    }

    public void setCode(String code)
    {
        this.code = code;
    }

    public String getCode()
    {
        return code;
    }

    public void setLoc(int loc)
    {
        this.loc = loc;
    }

    public int getLoc()
    {
        return loc;
    }

    public void generateRelations(ArrayList<JavaPackage> javaPackages)
    {
        this.relationsTo.clear();

        //Properties
        for (JavaProperty p : properties)
        {
            for (String s: p.extractRelations(javaPackages))
            {
                if (!this.fullName.equals(s))
                    this.relationsTo.add(new Relation(s, "Association"));
            }
        }

        //Methods
        for (JavaMethod m : methods)
        {
            for (String s : m.extractRelations(javaPackages))
            {
                if (!this.fullName.equals(s))
                   this.relationsTo.add(new Relation(s, "Association"));
            }
        }
        
         //Extends, Implements
        generateMoreRelations(javaPackages);
    }

    abstract void generateMoreRelations(ArrayList<JavaPackage> javaPackages);

    public String findFullType(String type, ArrayList<JavaPackage> javaPackages, boolean checkInternalClasses)
    {
        //Check internal
        if (checkInternalClasses == true && hasInternalEntityName(type))
            return getInternalEntityFullName(type);

        //Check imports
        for(String i: this.imports)
        {
            if(i.endsWith("." + type))
                return i;
        }

        for (JavaPackage jp : javaPackages)
        {
            if(jp.getName().equals(this.getPackageName()))
            {
                if(jp.hasEntityName(type))
                    return jp.getName().concat(".").concat(type);
            }
        }

        //fix import names
        for(String i: this.imports)
        {
            if(i.contains(".*"))
                i = i.substring(0, i.indexOf(".*"));
            else
                i = i.substring(0, i.lastIndexOf("."));
        }

        for(String i: this.imports)
        {
            for (JavaPackage jp : javaPackages)
            {
                if (i.startsWith(jp.getName()))
                    if(jp.hasEntityName(type))
                        return jp.getName().concat(".").concat(type);
            }
        }

        return type;
    }

    public void fix(ArrayList<JavaPackage> javaPackages)
    {
        //Fix properties
        loop: for (JavaProperty p : properties)
        {
            String type = p.getType();

            if (!type.contains(",") && type.contains(".")) //Property already in fulltype
                continue loop;

            int flag = 0;

            if (type.contains("<"))
            {
                flag = 1;
                type = type.substring(type.indexOf("<") + 1, type.indexOf(">"));
            }

            String fullType = "";
            if (type.contains(","))
            {
                String parts[] = type.split(",");
                for (int z = 0; z < parts.length; z++)
                {
                    fullType += this.findFullType(parts[z].trim(), javaPackages, true);
                    if (z < parts.length - 1)
                        fullType += ",";       
                }
            }
            else
                fullType = this.findFullType(type, javaPackages, true);

            if (!fullType.equals(type))
            {
                if (flag == 0)
                    p.setType(fullType);
                else
                if (flag == 1)
                    p.setType(p.getType().substring(0, p.getType().indexOf("<")) + "<" + fullType + ">");
            }
        }

        //Fix methods
        loop: for (JavaMethod m : methods)
        {
            //Fix method arguments
            for (JavaArgument arg: m.getArguments())
            {
                String type = arg.getType();
                if (!type.contains(",") && type.contains(".")) //Argument already in fulltype
                    continue;

                int flag = 0;

                if (type.contains("<"))
                {
                    flag = 1;
                    type = type.substring(type.indexOf("<") + 1, type.indexOf(">"));
                }

                String fullType = "";
                if (type.contains(","))
                {
                    String parts[] = type.split(",");
                    
                    for (int z = 0; z < parts.length; z++)
                    {
                        fullType += this.findFullType(parts[z].trim(), javaPackages, true);
                        if (z < parts.length - 1)
                            fullType += ",";
                    }
                }
                else
                    fullType = this.findFullType(type, javaPackages, true);

                if (!fullType.equals(type))
                {
                    if (flag == 0)
                        arg.setType(fullType);
                    else
                    if (flag == 1)
                        arg.setType(arg.getType().substring(0, arg.getType().indexOf("<")) + "<" + fullType + ">");
                }
            }

            //Fix method return type
            String type = m.getReturnType();
            if (!type.contains(",") && type.contains(".")) //Argument already in fulltype
                continue;

            int flag = 0;

            if (type.contains("<"))
            {
                flag = 1;
                type = type.substring(type.indexOf("<") + 1, type.indexOf(">"));
            }

            String fullType = "";
            if (type.contains(","))
            {
                String parts[] = type.split(",");

                for (int z = 0; z < parts.length; z++)
                {
                    fullType += this.findFullType(parts[z].trim(), javaPackages, true);
                    if (z < parts.length - 1)
                        fullType += ",";
                }
            }
            else
                fullType = this.findFullType(type, javaPackages, true);
            
            if (!fullType.equals(type))
            {
                if (flag == 0)
                    m.setReturnType(fullType);
                else
                if (flag == 1)
                    m.setReturnType(m.getReturnType().substring(0, m.getReturnType().indexOf("<")) + "<" + fullType + ">");
            }
        }

        for (JavaEntity e: internalEntities)
            e.fix(javaPackages);

        fixMore(javaPackages);
    }

    abstract void fixMore(ArrayList<JavaPackage> javaPackages);

    public String getInternalEntityFullName(String entityName)
    {
        for(JavaEntity e: internalEntities)
            if (entityName.equals(e.getName()))
                return e.getFullName();

        return entityName;
    }

    public boolean hasInternalEntityName(String entityName)
    {
        for(JavaEntity e: internalEntities)
            if (entityName.equals(e.getName()))
                return true;

        return false;
    }

    public boolean hasInternalEntityFullName(String entityFullName)
    {
        for(JavaEntity e: internalEntities)
            if (entityFullName.equals(e.getFullName()))
                return true;

        return false;
    }
    
    public boolean hasRelationTo(JavaEntity e)
    {
        for(Relation r: relationsTo)
            if (r.getRelatedEntity().equals(e.getFullName()))
                return true;

        return false;
    }

    public boolean hasEfferentCouplings()
    {
        if (this.relationsTo.isEmpty())
            return false;
        else
            return true;
    }

    public boolean hasAfferentCouplings()
    {
        if (this.relationsFrom.isEmpty())
            return false;
        else
            return true;
    }

    public void print(int indent)
    {
        
        System.out.println("Package: " + this.packageName + ". Class: " + this.getName() + " (" + this.getFullName() + ")");
        for (JavaProperty p: this.properties)
        {
            for (int i = 0; i < indent; i++)
                System.out.print("-->>");
            System.out.println(p.getName() + " " + p.getType());
        }

        for (String p: this.imports)
        {
            for (int i = 0; i < indent; i++)
                System.out.print("--==");
            System.out.println(p);
        }

        for (JavaMethod method: this.methods)
            method.print();

        for(Relation r : relationsTo)
            r.print();

        indent++;
        for (JavaEntity e: this.internalEntities)
            e.print(indent);

        System.out.println();
    }

    public boolean hasInternalEntities()
    {
        return this.internalEntities.size() > 0;
    }
    
}
