package jview;

import java.util.ArrayList;
import java.util.Arrays;

/**
 *
 * @author Lou
 */
public class Parser
{
    private ArrayList<JavaEntity> entities;
    private ArrayList<JavaPackage> packages;

    public Parser()
    {
        this.entities = new ArrayList<JavaEntity>();
        this.packages = new ArrayList<JavaPackage>();
        packages.add(new JavaPackage("default"));
    }

    public ArrayList<JavaEntity> getEntities()
    {
        return this.entities;
    }

    public ArrayList<JavaPackage> getPackages()
    {
        return this.packages;
    }

    public void parseCode(String code)
    {
        //System.out.println("Parsing new Java file...");

        //First remove comments and strings
        code = ParserUtil.prepareCode(code);
        String head = "";

        if(code.contains("{"))
            head = code.substring(0, code.indexOf("{"));
        
        //Split tokens with whitespaces
        String[] tokens = head.replaceAll(";", "").split("\\s+");

        String packageName = "default";
        ArrayList<String> imports = new ArrayList<String>();

        //Extract package_name and imports for all classes and interfaces used within the file
        //System.out.println("Extract package_name and imports for all classes and interfaces used within the file...");
        for (int i = 0; i < tokens.length; i++)
        {
            tokens[i] = tokens[i].trim();
            
            if (tokens[i].equals("package")) //Extract package name
            {
                if (i < tokens.length - 1)
                    packageName = tokens[i + 1];
            }
            else
            if  (tokens[i].equals("import")) //Extract imports
            {
                if (i < tokens.length - 1)
                {
                    if (!tokens[i + 1].equals("static"))
                        imports.add(tokens[i + 1]);
                    else
                        imports.add(tokens[i + 2]);
                }
            }
        }

        //Find package
        JavaPackage javaPackage = null;
        for(JavaPackage p: packages)
        {
            if(p.getName().equals(packageName))
            {
                javaPackage = p;
                break;
            }
        }

        if (javaPackage == null)
        {
            javaPackage = new JavaPackage(packageName);
            packages.add(javaPackage);
        }
        
        //ArrayList<JavaEntity> entities = new ArrayList<JavaEntity>();
        
        String text = "";
        
        //Otan mpainoume se block auksanetai kata ena otan bgainoume apo block meionetai kata ena
        int entity_counter = 0;
        
        int code_length = code.length();

        int loc = 0;
        //Extract main classes and main interfaces
        for (int i = 0; i < code_length; i++)
        {          
            switch (code.charAt(i))
            {
                case '{':
                    text += code.charAt(i);
                    
                    entity_counter++;
                    
                    break;
                
                case '}':
                    text += code.charAt(i);
                    
                    entity_counter--;
                    
                    if (entity_counter == 0)
                    {
                        String temp = text.substring(0, text.indexOf("{"));

                        if (temp.indexOf("class") != -1)
                        {
                            //System.out.println("Class found");
                            JavaClass jclass = ClassParser.parse(packageName, imports, text.trim(), "");
                            this.entities.add(jclass);
                            javaPackage.addEntity(jclass);
                        }
                        else
                        if (temp.indexOf("interface") != -1)
                        {
                            JavaInterface jinterface = InterfaceParser.parse(packageName, imports, text.trim(), "");
                            this.entities.add(jinterface);
                            javaPackage.addEntity(jinterface);
                        }
                            
                        text = "";
                    }
                    
                    break;
                
                case ';':
                    if (entity_counter == 0)
                        text = "";
                    else
                        text += code.charAt(i);
                    
                    break;
                 
                default: 
                    text += code.charAt(i);
            }
        }
    }

    public void fix()
    {
        for (JavaEntity entity: this.entities)
            entity.fix(this.packages);
    }

    public void generateRelationsTo()
    {
        for (JavaEntity entity: this.entities)
            entity.generateRelations(packages);
    }
    
    public void generateRelationsFrom()
    {
        for (JavaEntity entity: this.entities)
        {
            for (JavaEntity e: this.entities)
            {
                if (e.hasRelationTo(entity))
                    for(int i = 0; i<e.countRelationsTo(entity); i++)
                        entity.addRelationFrom(new Relation(e.getFullName(), "Association"));
            }
        }
    }
}
